import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';

import 'components/loading_widget.dart';
import 'providers/polyline_provider.dart';

class PolylineBuilder extends StatelessWidget {
  final Widget Function(Set<Polyline>) builder;
  // To update camera zoom to display whole polyline on map
  final GoogleMapController controller;
  // TODO: handle array of multiple from/to latlng
  final LatLng from;
  final LatLng to;

  static String MAPBOX_ACCESS_TOKEN;

  const PolylineBuilder(
      {Key key, @required this.builder, this.from, this.to, this.controller})
      : assert(builder != null),
        super(key: key);

  @override
  Widget build(BuildContext context) => Scaffold(
        body: Stack(
          children: [
            // Map
            FutureBuilder(
              future: PolylineProvider.getPolyline(from, to),
              builder: (context, snapshot) {
                if (controller != null)
                  _fitCameraToPolyline(controller, snapshot.data);

                return Stack(
                  children: [
                    builder(snapshot.data),
                    Visibility(
                        visible:
                            snapshot.connectionState == ConnectionState.waiting,
                        child: Container(
                          color: Colors.white.withOpacity(0.5),
                          alignment: Alignment.center,
                          child: LoadingWidget(),
                        ))
                  ],
                );
              },
            ),
          ],
        ),
      );

  void _fitCameraToPolyline(GoogleMapController controller, Set<Polyline> p) {
    if (p == null) return;

    double minLat = p.first.points.first.latitude;
    double minLong = p.first.points.first.longitude;
    double maxLat = p.first.points.first.latitude;
    double maxLong = p.first.points.first.longitude;
    p.forEach((poly) {
      poly.points.forEach((point) {
        if (point.latitude < minLat) minLat = point.latitude;
        if (point.latitude > maxLat) maxLat = point.latitude;
        if (point.longitude < minLong) minLong = point.longitude;
        if (point.longitude > maxLong) maxLong = point.longitude;
      });
    });
    controller.moveCamera(CameraUpdate.newLatLngBounds(
        LatLngBounds(
            southwest: LatLng(minLat, minLong),
            northeast: LatLng(maxLat, maxLong)),
        20));
  }
}
