import 'package:flutter/material.dart';
import 'package:google_maps_flutter/google_maps_flutter.dart';
import 'package:polyline/polyline.dart' as Poly;

class PolylineListResponseModel {
  PolylineListResponseModel({
    this.routes,
    this.waypoints,
    this.code,
    this.uuid,
  });

  List<Route> routes;
  List<Waypoint> waypoints;
  String code;
  String uuid;

  factory PolylineListResponseModel.fromJson(Map<String, dynamic> json) =>
      PolylineListResponseModel(
        routes: List<Route>.from(json["routes"].map((x) => Route.fromJson(x))),
        waypoints: List<Waypoint>.from(
            json["waypoints"].map((x) => Waypoint.fromJson(x))),
        code: json["code"],
        uuid: json["uuid"],
      );

  Map<String, dynamic> toJson() => {
        "routes": List<dynamic>.from(routes.map((x) => x.toJson())),
        "waypoints": List<dynamic>.from(waypoints.map((x) => x.toJson())),
        "code": code,
        "uuid": uuid,
      };

  // To Google Map Polyline
  Set<Polyline> toGoogleMapPolylineSet() {
    // Working with the first available route always
    final route = routes.first;
    // Decode geometry to get list of latitude & longitude
    final points =
        Poly.Polyline.Decode(encodedString: route.geometry, precision: 6)
            .decodedCoords;
    // Parse to list of LatLng
    final List<LatLng> latLngList =
        points.map((point) => LatLng(point[0], point[1])).toList();
    // Create random polyline id
    final polylineId = PolylineId(DateTime.now().microsecond.toString());
    // Create polyline instance
    final polyline = Polyline(
        polylineId: polylineId,
        points: latLngList,
        width: 5,
        color: const Color(0xFF669DF6));
    // Return set of polyline
    return {polyline};
  }
}

class Route {
  Route({
    this.weightName,
    this.weight,
    this.duration,
    this.distance,
    this.legs,
    this.geometry,
  });

  String weightName;
  double weight;
  double duration;
  double distance;
  List<Leg> legs;
  String geometry;

  factory Route.fromJson(Map<String, dynamic> json) => Route(
        weightName: json["weight_name"],
        weight: json["weight"].toDouble(),
        duration: json["duration"].toDouble(),
        distance: json["distance"].toDouble(),
        legs: List<Leg>.from(json["legs"].map((x) => Leg.fromJson(x))),
        geometry: json["geometry"],
      );

  Map<String, dynamic> toJson() => {
        "weight_name": weightName,
        "weight": weight,
        "duration": duration,
        "distance": distance,
        "legs": List<dynamic>.from(legs.map((x) => x.toJson())),
        "geometry": geometry,
      };
}

class Leg {
  Leg({
    this.steps,
    this.admins,
    this.duration,
    this.distance,
    this.weight,
    this.summary,
  });

  List<dynamic> steps;
  List<Admin> admins;
  double duration;
  double distance;
  double weight;
  String summary;

  factory Leg.fromJson(Map<String, dynamic> json) => Leg(
        steps: List<dynamic>.from(json["steps"].map((x) => x)),
        admins: List<Admin>.from(json["admins"].map((x) => Admin.fromJson(x))),
        duration: json["duration"].toDouble(),
        distance: json["distance"].toDouble(),
        weight: json["weight"].toDouble(),
        summary: json["summary"],
      );

  Map<String, dynamic> toJson() => {
        "steps": List<dynamic>.from(steps.map((x) => x)),
        "admins": List<dynamic>.from(admins.map((x) => x.toJson())),
        "duration": duration,
        "distance": distance,
        "weight": weight,
        "summary": summary,
      };
}

class Admin {
  Admin({
    this.iso31661,
  });

  String iso31661;

  factory Admin.fromJson(Map<String, dynamic> json) => Admin(
        iso31661: json["iso_3166_1"],
      );

  Map<String, dynamic> toJson() => {
        "iso_3166_1": iso31661,
      };
}

class Waypoint {
  Waypoint({
    this.distance,
    this.name,
    this.location,
  });

  double distance;
  String name;
  List<double> location;

  factory Waypoint.fromJson(Map<String, dynamic> json) => Waypoint(
        distance: json["distance"].toDouble(),
        name: json["name"],
        location: List<double>.from(json["location"].map((x) => x.toDouble())),
      );

  Map<String, dynamic> toJson() => {
        "distance": distance,
        "name": name,
        "location": List<dynamic>.from(location.map((x) => x)),
      };
}
